@@ -1,7 +1,7 @@
 /*  bam_ampliconclip.c -- loads amplicon primers from a BED file and cuts reads
                           from the 5' end.
 
-    Copyright (C) 2020-2021 Genome Research Ltd.
+    Copyright (C) 2020-2022 Genome Research Ltd.
 
     Authors: Andrew Whitwham <aw7@sanger.ac.uk>
              Rob Davies <rmd+git@sanger.ac.uk>
@@ -289,7 +289,7 @@ static int bam_trim_left(bam1_t *rec, bam1_t *rec_out, uint32_t bases,
     uint32_t *new_cigar;
     uint8_t *new_qual;
     size_t orig_l_aux = bam_get_l_aux(rec);
-    uint32_t i, j, odd_base = 0;
+    uint32_t i, j;
     uint32_t ref_remove = bases, qry_removed = 0, hardclip = 0;
     hts_pos_t new_pos = rec->core.pos;
     uint32_t cig_type, cig_op;
@@ -308,18 +308,6 @@ static int bam_trim_left(bam1_t *rec, bam1_t *rec_out, uint32_t bases,
     memcpy(&rec_out->core, &rec->core, sizeof(rec->core));
     memcpy(rec_out->data, rec->data, rec->core.l_qname);
 
-    if (clipping == hard_clip && bases >= rec->core.l_qseq) {
-        rec_out->core.l_qseq = 0;
-        rec_out->core.n_cigar = 0;
-
-        if (orig_l_aux)
-            memcpy(bam_get_aux(rec_out), orig_aux, orig_l_aux);
-
-        rec_out->l_data = bam_get_aux(rec_out) - rec_out->data + orig_l_aux;
-
-        return 0;
-    }
-
     // Modify CIGAR
     new_cigar = bam_get_cigar(rec_out);
 
@@ -355,6 +343,19 @@ static int bam_trim_left(bam1_t *rec, bam1_t *rec_out, uint32_t bases,
             qry_removed += ref_remove;
         }
     } else {
+        if (clipping == hard_clip) {
+
+            rec_out->core.l_qseq = 0;
+            rec_out->core.n_cigar = 0;
+
+            if (orig_l_aux)
+                memcpy(bam_get_aux(rec_out), orig_aux, orig_l_aux);
+
+            rec_out->l_data = bam_get_aux(rec_out) - rec_out->data + orig_l_aux;
+
+            return 0;
+        }
+
         qry_removed = rec->core.l_qseq;
     }
 
@@ -387,14 +388,13 @@ static int bam_trim_left(bam1_t *rec, bam1_t *rec_out, uint32_t bases,
 
     if (clipping == soft_clip) {
         qry_removed = 0; // Copy all the sequence and confidence values
-        odd_base = 1; // account for an odd number of bases
     }
 
     new_qual = bam_get_seq(rec_out) + (rec->core.l_qseq - qry_removed + 1) / 2;
     // Copy remaining SEQ
     if ((qry_removed & 1) == 0) {
         memcpy(bam_get_seq(rec_out), orig_seq + (qry_removed / 2),
-                (rec->core.l_qseq - qry_removed + odd_base) / 2);
+                (rec->core.l_qseq - qry_removed + 1) / 2); // +1 to account for odd numbers
     } else {
         uint8_t *in = orig_seq + qry_removed / 2;
         uint8_t *out = bam_get_seq(rec_out);
@@ -458,17 +458,6 @@ static int bam_trim_right(bam1_t *rec, bam1_t *rec_out, uint32_t bases,
     memcpy(&rec_out->core, &rec->core, sizeof(rec->core));
     memcpy(rec_out->data, rec->data, rec->core.l_qname);
 
-    if (clipping == hard_clip && bases >= rec->core.l_qseq) {
-        rec_out->core.l_qseq = 0;
-        rec_out->core.n_cigar = 0;
-
-        if (orig_l_aux)
-            memcpy(bam_get_aux(rec_out), orig_aux, orig_l_aux);
-
-        rec_out->l_data = bam_get_aux(rec_out) - rec_out->data + orig_l_aux;
-        return 0;
-    }
-
     // Modify CIGAR here
     new_cigar = bam_get_cigar(rec_out);
 
@@ -501,6 +490,19 @@ static int bam_trim_right(bam1_t *rec, bam1_t *rec_out, uint32_t bases,
         if (qry_removed > 0) j++;
         if (hardclip > 0 && (clipping == soft_clip || qry_removed == 0)) j++;
     } else {
+        if (clipping == hard_clip) {
+
+            rec_out->core.l_qseq = 0;
+            rec_out->core.n_cigar = 0;
+
+            if (orig_l_aux)
+                memcpy(bam_get_aux(rec_out), orig_aux, orig_l_aux);
+
+            rec_out->l_data = bam_get_aux(rec_out) - rec_out->data + orig_l_aux;
+
+            return 0;
+        }
+
         qry_removed = rec->core.l_qseq;
         j = 0;
         if (hardclip > 0 && clipping == soft_clip) j++;
